@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2018 GuardSquare NV
+ * Copyright (c) 2002-2019 Guardsquare NV
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -265,9 +265,45 @@ public class ConstantPoolEditor
 
 
     /**
-     * Finds or creates a InvokeDynamicConstant constant pool entry with the given
+     * Finds or creates a DynamicConstant constant pool entry with the given
      * class constant pool entry index and name and type constant pool entry
      * index.
+     * @return the constant pool index of the DynamicConstant.
+     */
+    public int addDynamicConstant(int     bootstrapMethodIndex,
+                                  int     nameAndTypeIndex,
+                                  Clazz[] referencedClasses)
+    {
+        int        constantPoolCount = targetClass.u2constantPoolCount;
+        Constant[] constantPool      = targetClass.constantPool;
+
+        // Check if the entry already exists.
+        for (int index = 1; index < constantPoolCount; index++)
+        {
+            Constant constant = constantPool[index];
+
+            if (constant != null &&
+                constant.getTag() == ClassConstants.CONSTANT_Dynamic)
+            {
+                DynamicConstant dynamicConstant = (DynamicConstant)constant;
+                if (dynamicConstant.u2bootstrapMethodAttributeIndex == bootstrapMethodIndex &&
+                    dynamicConstant.u2nameAndTypeIndex              == nameAndTypeIndex)
+                {
+                    return index;
+                }
+            }
+        }
+
+        return addConstant(new DynamicConstant(bootstrapMethodIndex,
+                                               nameAndTypeIndex,
+                                               referencedClasses));
+    }
+
+
+    /**
+     * Finds or creates an InvokeDynamicConstant constant pool entry with the
+     * given class constant pool entry index and name and type constant pool
+     * entry index.
      * @return the constant pool index of the InvokeDynamicConstant.
      */
     public int addInvokeDynamicConstant(int     bootstrapMethodIndex,
