@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2018 GuardSquare NV
+ * Copyright (c) 2002-2019 Guardsquare NV
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -23,7 +23,7 @@ package proguard.optimize.evaluation;
 import proguard.classfile.*;
 import proguard.classfile.attribute.*;
 import proguard.classfile.attribute.visitor.*;
-import proguard.classfile.constant.ClassConstant;
+import proguard.classfile.constant.*;
 import proguard.classfile.constant.visitor.ConstantVisitor;
 import proguard.classfile.instruction.*;
 import proguard.classfile.instruction.visitor.InstructionVisitor;
@@ -46,8 +46,9 @@ extends      SimplifiedVisitor
 implements   ClassVisitor,
              MemberVisitor,
              AttributeVisitor,
-             InstructionVisitor,
+             BootstrapMethodInfoVisitor,
              ConstantVisitor,
+             InstructionVisitor,
              ParameterVisitor
 {
     //*
@@ -92,6 +93,9 @@ implements   ClassVisitor,
 
     public void visitProgramClass(ProgramClass programClass)
     {
+        // Unmark the simple enum classes in bootstrap methods attributes.
+        programClass.attributesAccept(this);
+
         if ((programClass.getAccessFlags() & ClassConstants.ACC_ANNOTATION) != 0)
         {
             // Unmark the simple enum classes in annotations.
@@ -110,6 +114,13 @@ implements   ClassVisitor,
     public void visitAnyAttribute(Clazz clazz, Attribute attribute) {}
 
 
+    public void visitBootstrapMethodsAttribute(Clazz clazz, BootstrapMethodsAttribute bootstrapMethodsAttribute)
+    {
+        // Unmark the simple enum classes in all bootstrap methods.
+        bootstrapMethodsAttribute.bootstrapMethodEntriesAccept(clazz, this);
+    }
+
+
     public void visitCodeAttribute(Clazz clazz, Method method, CodeAttribute codeAttribute)
     {
         // Evaluate the method.
@@ -139,6 +150,59 @@ implements   ClassVisitor,
     }
 
 
+    // Implementations for BootstrapMethodInfoVisitor.
+
+    public void visitBootstrapMethodInfo(Clazz clazz, BootstrapMethodInfo bootstrapMethodInfo)
+    {
+        // Unmark the simple enum classes referenced in the method handle.
+        bootstrapMethodInfo.methodHandleAccept(clazz, this);
+
+        // Unmark the simple enum classes referenced in the method arguments.
+        bootstrapMethodInfo.methodArgumentsAccept(clazz, this);
+    }
+
+
+    // Implementations for ConstantVisitor.
+
+    public void visitAnyConstant(Clazz clazz, Constant constant) {}
+
+
+    public void visitStringConstant(Clazz clazz, StringConstant stringConstant)
+    {
+        // Unmark any simple enum class referenced in the string constant.
+        stringConstant.referencedClassAccept(complexEnumMarker);
+    }
+
+
+    public void visitMethodHandleConstant(Clazz clazz, MethodHandleConstant methodHandleConstant)
+    {
+        // Unmark the simple enum classes referenced in the method handle
+        // (through a reference constant).
+        methodHandleConstant.referenceAccept(clazz, this);
+    }
+
+
+    public void visitMethodTypeConstant(Clazz clazz, MethodTypeConstant methodTypeConstant)
+    {
+        // Unmark the simple enum classes referenced in the method type constant.
+        methodTypeConstant.referencedClassesAccept(referencedComplexEnumMarker);
+    }
+
+
+    public void visitAnyRefConstant(Clazz clazz, RefConstant refConstant)
+    {
+        // Unmark the simple enum classes referenced in the reference.
+        refConstant.referencedClassAccept(referencedComplexEnumMarker);
+    }
+
+
+    public void visitClassConstant(Clazz clazz, ClassConstant classConstant)
+    {
+        // Unmark any simple enum class referenced in the class constant.
+        classConstant.referencedClassAccept(complexEnumMarker);
+    }
+
+
     // Implementations for InstructionVisitor.
 
     public void visitSimpleInstruction(Clazz clazz, Method method, CodeAttribute codeAttribute, int offset, SimpleInstruction simpleInstruction)
