@@ -8,18 +8,15 @@ source ../buildscripts/functions.sh
 
 MAIN_CLASS=proguard.gui.ProGuardGUI
 
-# Make sure the ProGuard core has been compiled.
-if [ ! -d ../core/$OUT ]; then
+# Make sure ProGuard has been compiled.
+if [[ ! -d ../core/$OUT || ! -f "$PROGUARD_JAR" ]]; then
   ../core/build.sh || exit 1
 fi
 
 # Make sure ReTrace has been compiled.
-if [ ! -d ../retrace/$OUT ]; then
+if [[ ! -d ../retrace/$OUT || ! -f "$RETRACE_JAR" ]]; then
   ../retrace/build.sh || exit 1
 fi
 
-# Compile and package.
-export CLASSPATH=../core/$OUT:../retrace/$OUT
-
-compile   $MAIN_CLASS && \
+compile   $MAIN_CLASS "../core/$OUT:../retrace/$OUT" && \
 createjar "$PROGUARD_GUI_JAR" || exit 1
