@@ -8,13 +8,11 @@ source ../buildscripts/functions.sh
 
 MAIN_CLASS=proguard.retrace.ReTrace
 
-# Make sure the ProGuard core has been compiled.
-if [ ! -d ../core/$OUT ]; then
+# Make sure ProGuard has been compiled.
+if [[ ! -d ../core/$OUT ||
+      ! -f "$PROGUARD_JAR" ]]; then
   ../core/build.sh || exit 1
 fi
 
-# Compile and package.
-export CLASSPATH=../core/$OUT
-
-compile   $MAIN_CLASS && \
+compile   $MAIN_CLASS "../core/$OUT" && \
 createjar "$RETRACE_JAR" || exit 1
