@@ -19,13 +19,11 @@ if [ ! -f "$WTK_JAR" ]; then
   exit 1
 fi
 
-# Make sure the ProGuard core has been compiled.
-if [ ! -d ../core/$OUT ]; then
+# Make sure ProGuard has been compiled.
+if [[ ! -d ../core/$OUT ||
+      ! -f "$PROGUARD_JAR" ]]; then
   ../core/build.sh || exit 1
 fi
 
-# Compile and package.
-export CLASSPATH=../core/$OUT:$WTK_JAR
-
-compile   $MAIN_CLASS && \
+compile   $MAIN_CLASS "../core/$OUT:$WTK_JAR" && \
 updatejar "$PROGUARD_JAR" || exit 1
